from django import template
from django.utils.termcolors import colorize as dj_colorize

register = template.Library()


@register.filter
def colorize(value, fg):
    return dj_colorize(value, tuple(), fg=fg)



import logging
import re


class GenericAttributeFilter(logging.Filter):
    def __init__(self, name="", field="", regex=""):
        """
        Filter requests based on one of their attributes.

        >>> GenericAttributeFilter(field='foo', regex='aaa').field
        'foo'
        >>> GenericAttributeFilter(field='foo', regex='aaa').regex
        re.compile('aaa')

        :param name: Filter name
        :param field: Field name
        :param regex: Regular expression
        """
        self.field = field
        self._regex = regex
        self._compiled = None
        super().__init__(name)

    @property
    def regex(self):
        if not self._compiled:
            self._compiled = re.compile(self._regex)
        return self._compiled

    def filter(self, record):
        value = getattr(record.msg, self.field, None)
        if value is None:
            return True  # allow non-matching ones
        return self.regex.match(value)


class StatusCodeFilter(GenericAttributeFilter):
    def __init__(self, name="", status_codes=""):
        """
        Filter responses based on their status codes.

        >>> StatusCodeFilter().status_codes
        []
        >>> StatusCodeFilter(status_codes="404").status_codes
        [404]
        >>> StatusCodeFilter(status_codes="401,404").status_codes
        [401, 404]

        :param name: Filter name
        :param status_codes: Disallowed status codes as comma-separated string
        """
        self.status_codes = (
            len(status_codes) and list(map(int, status_codes.split(","))) or []
        )
        super().__init__(name)

    def filter(self, record):
        return not (
            hasattr(record.msg, "status_code")
            and record.msg.status_code in self.status_codes
        )


class PathFilter(GenericAttributeFilter):
    """
    Example configuration:
    {
        "()": "ddrr.filters.PathFilter",
        "regex": "\\/favicon.*",
    }
    """

    def __init__(self, **kwargs):
        super().__init__(field="path", **kwargs)



import logging

from django.apps import AppConfig
from django.conf import settings

from ddrr.formatters import DjangoTemplateRequestFormatter
from ddrr.formatters import DjangoTemplateResponseFormatter
from ddrr.loggers import request_logger
from ddrr.loggers import response_logger

logger = logging.getLogger(__name__)


class DDRRConfig(AppConfig):
    name = "ddrr"

    def ready(self):
        if "ddrr.middleware.DebugRequestsResponses" not in settings.MIDDLEWARE:
            logger.warning("DDRR middleware not configured")
            return

        def s(name, default):
            config = getattr(settings, "DDRR", None)
            if not config:
                return default
            return config.get(name, default)

        # get settings
        enable_requests = s("ENABLE_REQUESTS", True)
        enable_responses = s("ENABLE_RESPONSES", True)
        level = s("LEVEL", "DEBUG")
        pretty = s("PRETTY_PRINT", False)
        request_template_name = s(
            "REQUEST_TEMPLATE_NAME", "ddrr/default-request.html"
        )
        request_template = s("REQUEST_TEMPLATE", None)
        response_template_name = s(
            "RESPONSE_TEMPLATE_NAME", "ddrr/default-response.html"
        )
        response_template = s("RESPONSE_TEMPLATE", None)
        request_handler = s("REQUEST_HANDLER", logging.StreamHandler())
        response_handler = s("RESPONSE_HANDLER", logging.StreamHandler())
        colors = s("ENABLE_COLORS", True)
        limit_body = s("LIMIT_BODY", None)
        disable_django_server_log = s("DISABLE_DJANGO_SERVER_LOG", False)

        # set up request logger and handler
        request_handler.setLevel(level)
        request_logger.setLevel(level)
        request_logger.addHandler(request_handler)
        if not enable_requests:
            request_logger.disabled = False

        # set up response logger and handler
        response_handler.setLevel(level)
        response_logger.setLevel(level)
        response_logger.addHandler(response_handler)
        if not enable_responses:
            response_logger.disabled = False

        # set up request formatter
        request_formatter_kwargs = {
            "pretty": pretty,
            "colors": colors,
            "limit_body": limit_body,
        }
        if request_template:
            request_formatter_kwargs["template"] = request_template
        else:
            request_formatter_kwargs["template_name"] = request_template_name
        request_formatter = DjangoTemplateRequestFormatter(
            **request_formatter_kwargs
        )
        request_handler.setFormatter(request_formatter)

        # set up response formatter
        response_formatter_kwargs = {
            "pretty": pretty,
            "colors": colors,
            "limit_body": limit_body,
        }
        if response_template:
            response_formatter_kwargs["template"] = response_template
        else:
            response_formatter_kwargs["template_name"] = response_template_name
        response_formatter = DjangoTemplateResponseFormatter(
            **response_formatter_kwargs
        )
        response_handler.setFormatter(response_formatter)

        # disable django server log
        if disable_django_server_log:
            logging.getLogger("django.server").disabled = True



import textwrap

import attr
from django.utils.functional import cached_property

from ddrr.utils import collect_request_headers
from ddrr.utils import pretty_print


@attr.s
class RequestLogRecord:
    record = attr.ib()
    request = attr.ib()
    _formatter = attr.ib()

    @cached_property
    def headers(self):
        return collect_request_headers(self.request)

    @cached_property
    def body(self):
        try:
            content = self.request.body.decode("utf-8")
        except UnicodeDecodeError:
            content = str(self.request.body)
        # optionally pretty print
        if self._formatter.pretty:
            content = pretty_print(content, self.content_type)
        # optionally limit output
        if self._formatter.limit_body:
            content = textwrap.shorten(
                content, self._formatter.limit_body, placeholder="..."
            )
        return content

    @cached_property
    def method(self):
        return self.request.method

    @cached_property
    def path(self):
        return self.request.path

    @cached_property
    def query_string(self):
        query_params = self.query_params
        if not len(query_params):
            return ""
        return "?" + query_params.urlencode()

    @cached_property
    def query_params(self):
        return self.request.GET

    @cached_property
    def content_type(self):
        return self.headers.get("Content-Type", "")

    @classmethod
    def make(cls, record, formatter):
        return cls(record=record, request=record.msg, formatter=formatter)


@attr.s
class ResponseLogRecord:
    record = attr.ib()
    response = attr.ib()
    _formatter = attr.ib()

    @cached_property
    def headers(self):
        return dict(self.response.items())

    @cached_property
    def reason_phrase(self):
        return self.response.reason_phrase

    @cached_property
    def status_code(self):
        return self.response.status_code

    @cached_property
    def content(self):
        try:
            if self.response.streaming:
                content = "<streaming>"
            else:
                content = self.response.content.decode("utf-8")
        except UnicodeDecodeError:
            content = str(self.response.content)
        # optionally pretty print
        if self.content_type and self._formatter.pretty:
            content = pretty_print(content, self.content_type)
        # optionally limit output
        if self._formatter.limit_body:
            content = textwrap.shorten(
                content, self._formatter.limit_body, placeholder="..."
            )
        return content

    @cached_property
    def content_type(self):
        return self.headers.get("Content-Type")

    @classmethod
    def make(cls, record, formatter):
        return cls(record=record, response=record.msg, formatter=formatter)



import json
import re
from collections import OrderedDict
from xml.dom import minidom

import django

try:
    from lxml import etree
    from lxml.etree import XMLSyntaxError
except ImportError:
    etree = None
    XMLSyntaxError = None


SPECIAL_HEADERS = {
    header.lower(): header
    for header in [
        "A-IM",
        "ALPN",
        "AMP-Cache-Transform",
        "ARC-Authentication-Results",
        "ARC-Message-Signature",
        "ARC-Seal",
        "C-PEP-Info",
        "C-PEP",
        "Cal-Managed-ID",
        "CalDAV-Timezones",
        "CDN-Loop",
        "Content-ID",
        "Content-MD5",
        "DASL",
        "DAV",
        "Differential-ID",
        "Discarded-X400-IPMS-Extensions",
        "Discarded-X400-MTS-Extensions",
        "DKIM-Signature",
        "DL-Expansion-History",
        "DNT",
        "EDIINT-Features",
        "ETag",
        "Expect-CT",
        "HTTP2-Settings",
        "IM",
        "Include-Referred-Token-Binding-ID",
        "Jabber-ID",
        "List-ID",
        "Message-ID",
        "Message-ID",
        "MIME-Version",
        "MMHS-Acp127-Message-Identifier",
        "MMHS-Authorizing-Users",
        "MMHS-Codress-Message-Indicator",
        "MMHS-Copy-Precedence",
        "MMHS-Exempted-Address",
        "MMHS-Extended-Authorisation-Info",
        "MMHS-Handling-Instructions",
        "MMHS-Message-Instructions",
        "MMHS-Message-Type",
        "MMHS-Originator-PLAD",
        "MMHS-Originator-Reference",
        "MMHS-Other-Recipients-Indicator-CC",
        "MMHS-Other-Recipients-Indicator-To",
        "MMHS-Primary-Precedence",
        "MMHS-Subject-Indicator-Codes",
        "MT-Priority",
        "NNTP-Posting-Date",
        "NNTP-Posting-Host",
        "Optional-WWW-Authenticate",
        "Original-Message-ID",
        "OSCORE",
        "P3P",
        "PEP",
        "PICS-Label",
        "Received-SPF",
        "Resent-Message-ID",
        "SIO-Label-History",
        "SIO-Label",
        "SLUG",
        "Status-URI",
        "SubOK",
        "TCN",
        "TE",
        "TLS-Report-Domain",
        "TLS-Report-Submitter",
        "TLS-Required",
        "TTL",
        "UA-Color",
        "UA-Media",
        "UA-Pixels",
        "UA-Resolution",
        "UA-Windowpixels",
        "URI",
        "VBR-Info",
        "WWW-Authenticate",
        "X-ATT-DeviceId",
        "X-Correlation-ID",
        "X-PGP-Sig",
        "X-Request-ID" "X-Riferimento-Message-ID",
        "X-UA-Compatible",
        "X-UIDH",
        "X-WebKit-CSP",
        "X-XSS-Protection",
        "X400-MTS-Identifier",
    ]
}


def meta_to_header(header):
    """
    Normalize an HTTP header as it appears in META.

    >>> meta_to_header("FOO_BAR")
    'Foo-Bar'
    >>> meta_to_header("_")
    '-'

    :param header: Header name
    :return: Normalized header name
    """
    return "-".join(part.capitalize() for part in header.split("_"))


def collect_request_headers(request):
    """
    Given an HTTP request, return its headers as a dictionary.

    :param request: Request object
    :return: Dictionary of headers
    """
    # django 2.2 provides headers already formatted
    if django.VERSION >= (2, 2):
        headers = dict(request.headers.items())
    else:
        # older versions require you to jump through hoops... collect the
        # META keys that start with HTTP_, strip HTTP_ and normalize the
        # remainder to the common header format.
        headers = {
            (meta_to_header(header[5:])): value
            for header, value in request.META.items()
            if header.startswith("HTTP_")
        }
        # Content-Type and Content-Length don't appear as HTTP_
        if "CONTENT_TYPE" in request.META:
            headers["Content-Type"] = request.META["CONTENT_TYPE"]
        if "CONTENT_LENGTH" in request.META:
            headers["Content-Length"] = request.META["CONTENT_LENGTH"]

    out_headers = OrderedDict()
    # replace e.g. Www-Authenticate with WWW-Authenticate, etc.
    for header, value in headers.items():
        header = SPECIAL_HEADERS.get(header.lower(), header)
        out_headers[header] = value

    # sometimes Content-Length will be the empty string even though it was
    # not sent by the client, so remove it.
    if out_headers.get("Content-Length") == "":
        del out_headers["Content-Length"]

    return out_headers


def pretty_print_xml(content):
    """
    Pretty-print an XML string and return it.

    >>> pretty_print_xml('')
    ''
    >>> pretty_print_xml('<p></u>')
    '<p></u>'
    >>> pretty_print_xml('<p><div><b>hel</b>lo!</div></p>')
    '<p>\\n  <div><b>hel</b>lo!</div>\\n</p>\\n'

    :param content: XML string
    :return: Pretty-printed XML string
    """
    if etree:
        try:
            parser = etree.XMLParser(remove_blank_text=True)
            tree = etree.fromstring(content, parser)
            return etree.tostring(tree, encoding=str, pretty_print=True)
        except XMLSyntaxError:
            return content
    # noinspection PyBroadException
    try:
        return minidom.parseString(content).toprettyxml(indent="  ")
    except Exception:
        return content


def pretty_print_json(content):
    """
    Pretty-print a JSON string and return it.

    >>> pretty_print_json('')
    ''
    >>> pretty_print_json('foobar')
    'foobar'
    >>> pretty_print_json('{"foo":"bar"}')
    '{\\n  "foo": "bar"\\n}'
    >>> pretty_print_json('   {"foo"  : "bar"  }')
    '{\\n  "foo": "bar"\\n}'

    :param content: JSON string
    :return: Pretty-printed JSON string
    """
    try:
        data = json.loads(content)
        return json.dumps(data, indent=2)
    except json.decoder.JSONDecodeError:
        return content


PRETTY_PRINTERS = OrderedDict(
    (
        (r"/json", pretty_print_json),
        (r"/xml", pretty_print_xml),
    ),
)


def pretty_print(content, content_type):
    for regex, handler in PRETTY_PRINTERS.items():
        if re.search(regex, content_type):
            return handler(content)
    return content



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ddrr', 'ddrr.templatetags']

package_data = \
{'': ['*'], 'ddrr': ['templates/ddrr/*']}

install_requires = \
['Django>=2.2', 'attrs>=19.1']

extras_require = \
{'xml': ['lxml>=4.6.3,<5.0.0']}

setup_kwargs = {
    'name': 'ddrr',
    'version': '3.0.0',
    'description': 'Print request and response headers, body (with pretty-printing), etc.',
    'long_description': '# Django Debug Requests & Responses (DDRR)\n\nGet more out of your `runserver` development output! Print request and response\nheaders, body (with pretty-printing), etc.  Highly customizable! Supports\nDjango 2.2+ and 3.0+, with Python 3.7+.\n\n- Full request headers\n- The entire request body\n- Pretty-printing optional\n- Colored output\n- Super easy setup\n- No extra dependencies\n\nDDRR can also be used for general logging with some configuration of your own.\n\n## Installation\n\n1. ```\n   $ pip install ddrr\n   ```\n\n2. Add `"ddrr"` to `INSTALLED_APPS`\n\n3. Insert `"ddrr.middleware.DebugRequestsResponses"` first in `MIDDLEWARE`\n\n**Done!** When you run `runserver`, you\'ll now get the entire HTTP requests and\nresponses, including headers and bodies.\n\nIf you don\'t like the default output format, read on...\n\n## Customization\n\n```python\nDDRR = {\n    "ENABLE_REQUESTS": True,  # enable request logging\n    "ENABLE_RESPONSES": True,  # enable response logging\n    "LEVEL": "DEBUG",  # ddrr log level\n    "PRETTY_PRINT": False,  # pretty-print JSON and XML\n    "REQUEST_TEMPLATE_NAME": "ddrr/default-request.html",  # request log template name\n    "REQUEST_TEMPLATE": None,  # request log template string (overrides template name)\n    "RESPONSE_TEMPLATE_NAME": "ddrr/default-response.html",  # response log template name\n    "RESPONSE_TEMPLATE": None,  # response log template string (overrides template name)\n    "REQUEST_HANDLER": logging.StreamHandler(),  # request log handler\n    "RESPONSE_HANDLER": logging.StreamHandler(),  # response log handler\n    "ENABLE_COLORS": True,  # enable colors if terminal supports it\n    "LIMIT_BODY": None,  # limit request/response body output to X chars\n    "DISABLE_DJANGO_SERVER_LOG": False,  # disable default django server log\n}\n```\n\n### Template contexts\n\nIf you want to customize request or response templates, you can use the following values:\n\n- **Request template context:**\n  - `ddrr.body` - request body\n  - `ddrr.content_type` - request content type\n  - `ddrr.formatter` - the formatter\n  - `ddrr.headers` - mapping of header fields and values\n  - `ddrr.method` - request method\n  - `ddrr.path` - request path\n  - `ddrr.query_params` - query parameters\n  - `ddrr.query_string` - query string\n  - `ddrr.record` - the actual log record object\n  - `ddrr.request` - the actual request object\n- **Response template context:**\n  - `ddrr.content` - response content\n  - `ddrr.content_type` - response content type\n  - `ddrr.formatter` - the formatter\n  - `ddrr.headers` - mapping of header fields and values\n  - `ddrr.reason_phrase` - response reason phrase\n  - `ddrr.record` - the actual log record object\n  - `ddrr.response` - the actual response object\n  - `ddrr.status_code` - response status code\n\nFor example, this will log the method, path and body of each request, as well\nas the status code, reason phrase and content of each response:\n\n```python\nDDRR = {\n    "REQUEST_TEMPLATE": "{{ ddrr.method }} {{ ddrr.path }}\\n"\n                        "{{ ddrr.body }}",\n    "RESPONSE_TEMPLATE": "{{ ddrr.status_code }} {{ ddrr.reason_phrase }}\\n"\n                         "{{ ddrr.content }}",\n}\n```\n\n### Pretty-printing\n\nBy default, pretty-printing is disabled.  Set `DDRR["PRETTY_PRINT"]` to `True`\nto enable it.\n\nPretty-printing of JSON requires no external dependency.\n\nPretty-printing of XML uses `minidom` by default and doesn\'t require any extra\ndependency. If you want to use `lxml` instead, which is slightly better at\npretty-printing XML, you can install that using `pip install ddrr[xml]`.\n\n## How it works internally\n\nThe middleware `ddrr.middleware.DebugRequestsResponses` sends the entire\nrequest object as the message to `ddrr-request-logger`.  This logger has been\nconfigured to use `ddrr.formatters.DjangoTemplateRequestFormatter` which\ninternally uses Django\'s built-in template engine to format the request into\nhuman-readable form. By default, this is shown in your console output, but you\ncan easily configure it to log it to a file, Logstash, or anything else.\n\n## Similar projects\n\n- [Django Debug Toolbar](https://django-debug-toolbar.readthedocs.io)\n\n## Development and contributions\n\nPR\'s are always welcome!\n\nFor hacking on DDRR, make sure you are familiar with:\n\n- [Black](https://github.com/ambv/black)\n- [Flake8](http://flake8.pycqa.org/)\n- [Poetry](https://poetry.eustace.io/)\n- [pre-commit](https://github.com/pre-commit/pre-commit)\n- [pytest](https://docs.pytest.org)\n\nInstall dependencies and set up the pre-commit hooks.\n\n```\n$ poetry install\n$ pre-commit install\n```\n\nThe pre-commit hooks will, among other things, run Flake8 on the code base and\nBlack to make sure the code style is consistent across all files.  Check out\n[`.pre-commit-config.yaml`](.pre-commit-config.yaml) for details.\n',
    'author': 'Deniz Dogan',
    'author_email': 'denizdogan@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/denizdogan/django-debug-requests-responses',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from ddrr.loggers import request_logger
from ddrr.loggers import response_logger


class DebugRequestsResponses:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        request_logger.debug(request)
        response = self.get_response(request)
        response_logger.debug(response)
        return response



import logging

from django.core.management.color import supports_color
from django.template import Context
from django.template import RequestContext
from django.template import Template
from django.template.loader import get_template

from ddrr.records import RequestLogRecord
from ddrr.records import ResponseLogRecord


class DjangoTemplateRequestFormatter(logging.Formatter):
    # noinspection PyMissingConstructor
    def __init__(
        self,
        *,
        template_name=None,
        template=None,
        pretty=False,
        limit_body=None,
        colors=True,
    ):
        if not template_name and not template:
            raise RuntimeError(
                "DjangoTemplateRequestFormatter requires a template_name or "
                "template setting"
            )
        self._template_name = template_name
        self._template = template
        self.pretty = pretty
        self.limit_body = limit_body
        self.colors = colors and supports_color()

    @property
    def template(self):
        # lazy loading of template to avoid AppRegistryNotReady
        return (
            # circumvent autoescape being forced onto the template context by
            # extracting the source and creating a separate Template object
            Template(get_template(self._template_name).template.source)
            if self._template_name
            else Template(self._template)
        )

    def format(self, record):
        ddrr = RequestLogRecord.make(record, self)
        ctx = RequestContext(
            request=record.msg,
            dict_={"ddrr": ddrr, "record": record, "formatter": self},
            autoescape=False,
        )
        return self.template.render(ctx)


class DjangoTemplateResponseFormatter(logging.Formatter):
    # noinspection PyMissingConstructor
    def __init__(
        self,
        *,
        template_name=None,
        template=None,
        pretty=False,
        limit_body=None,
        colors=True,
    ):
        if not template_name and not template:
            raise RuntimeError(
                "DjangoTemplateResponseFormatter requires a template_name or "
                "template setting"
            )
        self._template_name = template_name
        self._template = template
        self.pretty = pretty
        self.limit_body = limit_body
        self.colors = colors and supports_color()

    @property
    def template(self):
        # lazy loading of template to avoid AppRegistryNotReady
        return (
            # circumvent autoescape being forced onto the template context by
            # extracting the source and creating a separate Template object
            Template(get_template(self._template_name).template.source)
            if self._template_name
            else Template(self._template)
        )

    def format(self, record):
        ddrr = ResponseLogRecord.make(record, self)
        ctx = Context(
            dict_={"ddrr": ddrr, "record": record, "formatter": self},
            autoescape=False,
        )
        return self.template.render(ctx)
